##################### generated by xml-casa (v2) from importuvfits.xml ##############
##################### 264135ac2cd552a4f2b3991e0ce23596 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatasks import importuvfits as _importuvfits_t
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _importuvfits:
    """
    importuvfits ---- Convert a UVFITS file to a CASA visibility data set

    Convert a UVITS file to a CASA visiblity data set.
    Don't forget to flag autocorrelations using taskname flagdata, autocorr = true
    

    --------- parameter descriptions ---------------------------------------------

    fitsfile      Name of input UV FITS file
    vis           Name of output visibility file (MS)
    antnamescheme VLA/EVLA/CARMA only; \'new\' or \'old\'; \'VA04\' or \'04\' for VLA ant 4

    --------- examples -----------------------------------------------------------

    
    Convert a UVFITS file to a CASA visibility data set:
    
    Keyword arguments:
    fitsfile -- Name of input UV FITS file
    default = none; example='3C273XC1.fits'
    vis -- Name of output visibility file (MS)
    default = none; example: vis='3C273XC1.ms'
    antnamescheme -- Naming scheme for VLA/JVLA/CARMA antennas
    default = old;
    old: Antenna name is a number, '04'
    This option exists for backwards compatibility
    but can lead to ambiguous results when antenna
    indices are used for data selection.
    new: Antenna name is not a number, e.g., 'VA04' or 'EA04'
    With this scheme, data selection via
    antenna names and indices is non-ambiguous.
    
    Note: Don't forget to flag autocorrections using
    taskname flagdata, autocorr = true
    
    


    """

    _info_group_ = """import/export"""
    _info_desc_ = """Convert a UVFITS file to a CASA visibility data set"""

    __schema = {'fitsfile': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'vis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'antnamescheme': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'old', 'new' ]}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 13 + 4
        output = [ ]
        addon = ''
        first_addon = True
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __fitsfile_dflt( self, glb ):
        return ''

    def __fitsfile( self, glb ):
        if 'fitsfile' in glb: return glb['fitsfile']
        return ''

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __antnamescheme_dflt( self, glb ):
        return 'old'

    def __antnamescheme( self, glb ):
        if 'antnamescheme' in glb: return glb['antnamescheme']
        return 'old'



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __fitsfile_inp(self):
        description = 'Name of input UV FITS file'
        value = self.__fitsfile( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'fitsfile': value},{'fitsfile': self.__schema['fitsfile']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('fitsfile',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __vis_inp(self):
        description = 'Name of output visibility file (MS)'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __antnamescheme_inp(self):
        description = 'VLA/EVLA/CARMA only; \'new\' or \'old\'; \'VA04\' or \'04\' for VLA ant 4'
        value = self.__antnamescheme( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'antnamescheme': value},{'antnamescheme': self.__schema['antnamescheme']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('antnamescheme',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'fitsfile' in glb: del glb['fitsfile']
        if 'vis' in glb: del glb['vis']
        if 'antnamescheme' in glb: del glb['antnamescheme']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# importuvfits -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__fitsfile_inp( )
        self.__vis_inp( )
        self.__antnamescheme_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,file=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = None
        if file is None:
            if os.path.isfile("importuvfits.last"):
                filename = "importuvfits.last"
        elif isinstance(file, str):
            if os.path.isfile(file):
                filename = file
        if filename is not None:
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file, setting defaults instead...")
            self.set_global_defaults( )

    def __call__( self, fitsfile=None, vis=None, antnamescheme=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('importuvfits.pre')
        _postfile = os.path.realpath('importuvfits.last')
        _return_result_ = None
        _arguments = [fitsfile,vis,antnamescheme]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if fitsfile is not None: local_global['fitsfile'] = fitsfile
            if vis is not None: local_global['vis'] = vis
            if antnamescheme is not None: local_global['antnamescheme'] = antnamescheme

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['fitsfile'] = self.__fitsfile( local_global )
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['antnamescheme'] = self.__antnamescheme( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['fitsfile'] = self.__fitsfile( self.__globals_( ) )
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['antnamescheme'] = self.__antnamescheme( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-13s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importuvfits( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _return_result_ = _importuvfits_t( _invocation_parameters['fitsfile'],_invocation_parameters['vis'],_invocation_parameters['antnamescheme'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('importuvfits')
            casalog.post("Exception Reported: Error in importuvfits: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            _return_result_ = False
        try:
            os.rename(_prefile,_postfile)
        except: pass
        return _return_result_

importuvfits = _importuvfits( )

